import { Method } from 'axios';
import { NCPAuthKeyType } from '../types/auth_types';
import { ApiClientResponse } from '../types/return_types';
export declare class ApiClient {
    /**
     * The account access Key for API authentication
     *
     * @access public
     * @readonly
     * @type {NCPAuthKeyType}
     * @memberof ApiClient
     */
    readonly ncpAuthKey: NCPAuthKeyType;
    /**
     * The session expressed by Wrapped AxiosInstance
     *
     * @access private
     * @type {AxiosInstance}
     * @memberof ApiClient
     */
    private client;
    /**
     * Creates an instance of ApiClient.
     *
     * @param {NCPAuthKeyType} ncpAuthKey NCP API authentication data for using API
     * @param {string} baseURL The baseURL of each session ( based on service )
     * @param {(number | undefined)} timeout Timeout configuration ( millisecond )
     * @memberof ApiClient
     */
    constructor(ncpAuthKey: NCPAuthKeyType, baseURL: string, timeout?: number);
    /**
     * Executing request and wrapping it with ApiClientResponse
     *  will return ApiClientResponse if success, with data else with error message
     *
     * @template T - Data type matched with each API's response
     * @async
     * @access public
     * @param {ApiRequest} apiRequest - ApiRequest configs for request
     * @returns {Promise<ApiClientResponse<T>>} return Promise response of wrapped with error handling
     * @memberof ApiClient
     */
    request<T>(apiRequest: ApiRequest): Promise<ApiClientResponse<T>>;
    /**
     * Execute request and return Promise with error or response if success
     *  Wrapping axios-request with Promise's resolve / reject for error handling
     *
     * @template T - Data type matched with each API's response
     * @access private
     * @param {ApiRequest} apiRequest - ApiRequest configs for request
     * @returns {Promise<T>} return Promise response of http request with current ApiRequest configs and handle errors
     * @memberof ApiClient
     */
    private createRequest;
    /**
     * Return configured AxiosInstance's request execution
     *  Before launch axios-request, launch url validation if invalid, throw Error else return AxiosResponses
     *
     * @access private
     * @param {ApiRequest} apiRequest - ApiRequest configs for request
     * @returns {Promise<AxiosResponse>} return Promise response of http request with current ApiRequest configs and handle errors
     * @memberof ApiClient
     */
    private urlRequest;
    /**
     * URL validation using regex
     * @access private
     * @param url
     * @returns {boolean}
     * @memberof ApiClient
     */
    private validateURL;
}
/**
 * ApiRequest configs for http request
 *
 * @interface ApiRequest
 * @member {string} path URI path for current request
 * @member {Method} member Http Method for current request
 * @member {{[key:string]: string}} headers Header for current request
 * @member {({[key:string]: any} | undefined)} body Body for current request
 */
export interface ApiRequest {
    path: string;
    method: Method;
    headers: {
        [key: string]: string;
    };
    body?: {
        [key: string]: any;
    };
}
